@@ -101,6 +101,14 @@ class Stock:
 	def get_month_data(self, num=1):
 		df = Ticker(self.ticker).get_data('1d', f'{num}mo')
 		df.index = pd.to_datetime(df.index)
+		for i in range(2, df.shape[0]):
+			df.loc[df.index[i], '2_sma'] = sum([float(i) for i in df.iloc[i-2:i]['Close']])/2
+		for i in range(9, df.shape[0]):
+			df.loc[df.index[i], '9_sma'] = sum([float(i) for i in df.iloc[i-9:i]['Close']])/9
+		try:
+			df = df[['Close', 'Open', 'High', 'Low', '2_sma', '9_sma', 'Volume']]
+		except:
+			pass
 		return df
 
 	def add_target_prices(self, new_target_prices):
@@ -174,33 +182,50 @@ class Stock:
 			# Find the website of the stock and go to its information page
 
 		def _basic_stats(ticker):
-			# Market cap, avg volume, price, chart, 
+			# Market cap, avg volume, financial information
 			BASE_URL = f'https://finance.yahoo.com/quote/{ticker}/key-statistics?p={ticker}'
 			soup = _get_soup(BASE_URL)
 
-			with open('output1.html', 'w', encoding='utf-8') as file:
-				file.write(str(soup))
+			info = []
 
 			# Financial highlights
 			div = soup.find('div', {'class': 'Mb(10px) Pend(20px) smartphone_Pend(0px)'})
-			print([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])
+			info.append([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])
 
 			# Trading Information
 			div = soup.find('div', {'class': 'Pstart(20px) smartphone_Pstart(0px)'})
-			print([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])
+			info.append([(i.find('h3', {'class': 'Mt(20px)'}), i.find('tbody').find_all('tr')) for i in div.find_all('div', {'class': 'Pos(r) Mt(10px)'})])
+
+			info_df = pd.DataFrame(info, columns={'Label', 'Value'})
 
 			# Income Statement
+			nerd_info = []
 			BASE_URL = f'https://finance.yahoo.com/quote/{ticker}/financials?p={ticker}'
 			soup = _get_soup(BASE_URL)
+			rows = soup.find_all('div', {'data-test': 'fin-row'})
+			for row in rows:
+				title = row.find('div', {'class': 'D(tbc) Ta(start) Pend(15px)--mv2 Pend(10px) Bxz(bb) Py(8px) Bdends(s) Bdbs(s) Bdstarts(s) Bdstartw(1px) Bdbw(1px) Bdendw(1px) Bdc($seperatorColor) Pos(st) Start(0) Bgc($lv2BgColor) fi-row:h_Bgc($hoverBgColor) Pstart(15px)--mv2 Pstart(10px)'})
+				data = row.find('div', {'data-test': 'fin-col'})
+				nerd_info.append([title.get_text(), data.get_text()])
+			
 			# Balance Sheet
 			BASE_URL = f'https://finance.yahoo.com/quote/{ticker}/balance-sheet?p={ticker}'
 			soup = _get_soup(BASE_URL)
-			pass
+			rows = soup.find_all('div', {'data-test': 'fin-row'})
+			for row in rows:
+				title = row.find('D(ib) Fw(b) Ta(start) Px(15px)--mv2 Px(10px) W(247px)--mv2 W(222px) Bxz(bb) Bdendw(1px) Bdstartw(1px) Bdbw(1px) Bdends(s) Bdstarts(s) Bdbs(s) Bdc($seperatorColor) Py(6px) Pos(st) Start(0) Bgc($lv2BgColor)')
+				data = row.find('div', {'data-test': 'fin-col'})
+				nerd_info.append([title.get_text(), data.get_text()])
 
-		def _price_target(self, exchange='NASDAQ'): # Automatically find correct stock exchange
+			nerd_df = pd.DataFrame(nerd_info, columns={'Label', 'Value'})
+			
+			return info_df, nerd_df
+
+		def _price_target(ticker, exchange='NASDAQ'): # Automatically find correct stock exchange
+			BASE_URL = f'https://www.marketbeat.com/stocks/{exchange}/{ticker}/price-target/'
 			soup = _get_soup(BASE_URL)
-			table = soup.find('table', {'class': "scroll-table"})
-			# price_target = soup.find('table', {'class': 'scroll-table'})
+			
+			price_target = soup.find('table', {'class': 'scroll-table'})
 			_pattern = re.compile(r'Price Target: \$\d{1,3}\.\d\d')
 			price_target = _find_match(_pattern, table.get_text()).group(0)
 			_pattern = re.compile(r'\d{1,3}\.\d\d\% \w{6,8}')
@@ -239,17 +264,19 @@ class Stock:
 				indictator = ' '.join(i.split()[:-3])
 				df_data.append((indictator, signal, strength, direction))
 			df = pd.DataFrame(df_data, columns=['Indictator', 'Signal', 'Strength', 'Direction'])
-			print(df.head())
+			return df, titles
 
-		def _ta_indictators(self, exchange='NASDAQ'): # Loads wrong page. Beta, RSI history, above/below 9 SMA, above/below 180 SMA, volatility, rel volume
-			BASE_URL = f'https://www.tradingview.com/symbols/{exchange}-{ticker}/technicals/'
+		def _ta_indictators(self): # Loads wrong page. Beta, RSI history, above/below 9 SMA, above/below 180 SMA, volatility, rel volume
+			BASE_URL = f'https://www.tradingview.com/symbols/{ticker}/technicals/'
 			soup = _get_soup(BASE_URL)
 
+			with open('output1.html', 'w', encoding='utf-8') as file:
+				file.write(str(soup))
+
 			# Buy or sell (Summary, Oscillators, Moving Averages)
 			s = soup.find_all('div', {'class': 'speedometerWrapper-1SNrYKXY'})
-			print(s)
-
-			# Oscillators
+			
+			# Oscillators, this is bugged
 			oscillators = soup.find('div', {'class': 'container-2w8ThMcC tableWithAction-2OCRQQ8y'})
 			# with open('output1.html', 'w', encoding='utf-8') as file:
 			# 	file.write(str(soup.prettify('utf-8')))
@@ -279,17 +306,17 @@ class Stock:
 			soup = _get_soup(BASE_URL)
 
 			links = soup.find_all('a', {'class': 'js-content-viewer wafer-caas Fw(b) Fz(18px) Lh(23px) LineClamp(2,46px) Fz(17px)--sm1024 Lh(19px)--sm1024 LineClamp(2,38px)--sm1024 mega-item-header-link Td(n) C(#0078ff):h C(#000) LineClamp(2,46px) LineClamp(2,38px)--sm1024 not-isInStreamVideoEnabled'})
-			print([(link.get_text(), str('yahoo.com' + link['href'])) for link in links])
+			news = [(link.get_text(), str('yahoo.com' + link['href'])) for link in links]
 
 			BASE_URL = f'https://finance.yahoo.com/quote/{ticker}/press-releases?p={ticker}'
 			soup = _get_soup(BASE_URL)
 
 			links = soup.find_all('a', {'class': 'js-content-viewer wafer-caas Fw(b) Fz(18px) Lh(23px) LineClamp(2,46px) Fz(17px)--sm1024 Lh(19px)--sm1024 LineClamp(2,38px)--sm1024 mega-item-header-link Td(n) C(#0078ff):h C(#000) LineClamp(2,46px) LineClamp(2,38px)--sm1024 not-isInStreamVideoEnabled'})
-			print([(link.get_text(), str('yahoo.com' + link['href'])) for link in links])
+			press_releases = [(link.get_text(), str('yahoo.com' + link['href'])) for link in links]
 			# Look for keywords in the news? Any showcases, Investor/analyst days, Analyst revisions, Management transitions
 			# Product launches, Significant stock buyback changes
 
-			return df
+			return df, news, press_releases
 
 		def _financials(ticker): # OMEGALUL
 			# Displaying all information. Could leave this as a dictionary.
@@ -321,6 +348,7 @@ class Stock:
 
 			labels = soup.find_all('td', {'class': 'snapshot-td2-cp'})
 			values = soup.find_all('td', {'class': 'snapshot-td2'})
+
 			return labels[16].get_text(), values[16].get_text(), labels[22].get_text(), values[22].get_text()
 
 
@@ -348,8 +376,7 @@ class Stock:
 			sectors = td.find_all('a', {'class': 'tab-link'})
 			sector_urls = ([str('https://finviz.com/' + i['href']) for i in sectors])
 			for i in sector_urls: # Find stocks with similar P/E ratios and market cap, then track difference in performance
-				print(i)
-				print('')
+				pass
 
 		def _etfs(ticker):
 			BASE_URL = f'https://etfdb.com/stock/{ticker}/'
@@ -380,30 +407,26 @@ class Stock:
 			return tweets
 
 		def _catalysts(ticker): # Returns date of showcases, FDA approvals, earnings, etc
+			df_data = []
 			# Earnings date: 
 			BASE_URL = f'https://finance.yahoo.com/quote/{ticker}?p={ticker}&.tsrc=fin-srch'
 			soup = _get_soup(BASE_URL)
-
+			
+			# Convert to datetime
 			earnings_date = soup.find('td', {'data-test': 'EARNINGS_DATE-value'})
-			print(f'Next earnings date: {earnings_date.get_text()}')
+			earnings_date = f'Next earnings date: {earnings_date.get_text()}'
 
 			# FDA approvals
 			BASE_URL = 'https://www.rttnews.com/corpinfo/fdacalendar.aspx'
 			soup = _get_soup(BASE_URL)
 
-			df_data = []
 			company = soup.find_all('div', {'data-th': 'Company Name'})
-			print(company[0].get_text())
 
 			events = soup.find_all('div', {'data-th': 'Event'})
-			print(events[0].get_text())
 
 			outcome = soup.find_all('div', {'data-th': 'Outcome'})
-			if outcome[0]:
-				print(outcome[0].get_text())
 
 			dates = soup.find_all('span', {'class': 'evntDate'})
-			print([date.get_text() for date in dates])
 
 			for i in range(len(company)):
 				if outcome[i]:
@@ -425,26 +448,17 @@ class Stock:
 			BASE_URL = f'https://money.cnn.com/quote/shareholders/shareholders.html?symb={ticker}&subView=institutional'
 			soup = _get_soup(BASE_URL)
 
-			# Latest institutional activity
-			table = soup.find('table', {'class', 'wsod_dataTable wsod_dataTableBig'})
-			rows = table.find_all('tr')
-			print('Recent large purchases:')
-			for row in rows:
-				date = row.find('td', {'class': 'wsod_activityDate'})
-				info = row.find('td', {'class': 'wsod_activityDetail'})
-				print(date.get_text(), info.get_text()) # Could make a data frame
-
-			# Top 10 Owners of self.{Ticker}
+			# Top 10 Owners
+			df_data = []
 			table = soup.find('table', {'class': 'wsod_dataTable wsod_dataTableBig wsod_institutionalTop10'})
 			rows = table.find_all('tr')[1:]
-			df_data = []
 			for row in rows:
 				data = row.find_all('td')
 				df_data.append([i.get_text() for i in data])
 
 			owners_df = pd.DataFrame(df_data, columns=['Stockholder', 'Stake', 'Shares owned', 'Total value($)', 'Shares bought / sold', 'Total change'])
 
-			# Top 10 Mutual Funds Holding self.{Ticker}
+			# Top 10 Mutual Funds Holding 
 			table = soup.find_all('table', {'class': 'wsod_dataTable wsod_dataTableBig wsod_institutionalTop10'})[1]
 			rows = table.find_all('tr')[1:]
 			df_data = []
@@ -454,21 +468,20 @@ class Stock:
 
 			mutual_funds_df = pd.DataFrame(df_data, columns=['Stockholder', 'Stake', 'Shares owned', 'Total value($)', 'Shares bought / sold', 'Total change'])
 
+			# Recent instutional investments, from a better source this time
 			BASE_URL = f'https://fintel.io/so/us/{ticker}'
 			soup = _get_soup(BASE_URL)
 			table = soup.find('table', {'id': 'transactions'})
-
 			rows = table.find_all('tr')
 			df_data = []
 			for row in rows[1:]:
 				date, form, investor, _, opt, avgshareprice, shares, shareschanged, value, valuechanged, _, _, _ = [i.get_text() for i in row.find_all('td')]
 				df_data.append([date, form, investor, opt, avgshareprice, shares, shareschanged, value, valuechanged])
-
 			recent_purchases_df = pd.DataFrame(df_data, columns=['Date', 'Form', 'Investor', 'Opt', 'Avg Share Price',
 				'Shares', 'Shares Changed (%)', 'Value ($1000)', 'Value Changed (%)'])
 			recent_purchases_df = recent_purchases_df.set_index('Date').sort_index(ascending=False)
 
-			return owners_df, mutual_funds_df, recent_purchases_df.tail()
+			return owners_df, mutual_funds_df, recent_purchases_df.tail(), 
 	
 		# Have not decided how I want to format returning all of this information
 		return _get_summary(self.ticker), _basic_stats(self.ticker), _price_target(self.ticker), _price_predictions(self.ticker), 
@@ -479,16 +492,6 @@ class Stock:
 	def __str__(self):
 		return self.ticker
 
-
-# 	for i in range(2, ticker.shape[0]):
-# 		ticker.loc[ticker.index[i], '2_sma'] = sum([float(i) for i in ticker.iloc[i-2:i]['Close']])/2
-# 	for i in range(9, ticker.shape[0]):
-# 		ticker.loc[ticker.index[i], '9_sma'] = sum([float(i) for i in ticker.iloc[i-9:i]['Close']])/9
-# 	try:
-# 		ticker = ticker[['Close', 'High', 'Low', '2_sma', '9_sma', 'Volume']]
-# 	except:
-# 		pass
-
 print('Welcome to PyTrading!')
 
 if __name__ == '__main__':
